@@ -1,4 +1,4 @@
-/* dict.c -- hash-table based dictionaries ($Revision: 1.1.1.1 $) */
+/* dict.c -- linked list based dictionaries */
 
 #include "es.h"
 #include "gc.h"
@@ -15,7 +15,7 @@
 static unsigned long strhash2(const char *str1, const char *str2) {
 
 #define	ADVANCE() { \
-		if ((c = *s++) == '\0') { \
+		if ((c = *s++) == '\0') \
 			if (str2 == NULL) \
 				break; \
 			else { \
@@ -24,7 +24,6 @@ static unsigned long strhash2(const char *str1, const char *str2) {
 				if ((c = *s++) == '\0') \
 					break; \
 			} \
-		} \
 	}
 
 	int c;
@@ -54,7 +53,7 @@ static unsigned long strhash(const char *str) {
  * data structures and garbage collection
  */
 
-DefineTag(Dict, static);
+static Tag DictTag;
 
 typedef struct {
 	char *name;
@@ -95,6 +94,8 @@ static size_t DictScan(void *p) {
 	return offsetof(Dict, table[dict->size]);
 }
 
+static DefineTag(Dict);
+
 
 /*
  * private operations
@@ -202,6 +203,18 @@ extern void dictforall(Dict *dp, void (*proc)(void *, char *, void *), void *arg
 	RefEnd2(argp, dict);
 }
 
+static Boolean streq2(const char *s, const char *t1, const char *t2) {
+	int c;
+	assert(s != NULL && t1 != NULL && t2 != NULL);
+	while ((c = *t1++) != '\0')
+		if (c != *s++)
+			return FALSE;
+	while ((c = *t2++) != '\0')
+		if (c != *s++)
+			return FALSE;
+	return TRUE;
+}
+
 /* dictget2 -- look up the catenation of two names (such a hack!) */
 extern void *dictget2(Dict *dict, const char *name1, const char *name2) {
 	Assoc *ap;
