@@ -1,35 +1,20 @@
-/* eval.c -- evaluation of lists and trees ($Revision: 1.11 $) */
+/* eval.c -- evaluation of lists and trees */
 
 #include "es.h"
 
-static noreturn failexec(char *file, List *args) {
-	List *fn;
-	assert(gcisblocked());
-	fn = varlookup("fn-%exec-failure", NULL);
-	if (fn != NULL) {
-		int olderror = errno;
-		Ref(List *, list, append(fn, mklist(mkterm(file, NULL), args)));
-		RefAdd(file);
-		gcenable();
-		RefRemove(file);
-		eval(list, NULL, 0);
-		RefEnd(list);
-		errno = olderror;
-	}
-	eprint("%s: %s\n", file, strerror(errno));
-	exit(1);
-}
+extern int execve(char *name, char **argv, char **envp);
 
 /* forkexec -- fork (if necessary) and exec */
-static List *forkexec(char *file, List *list, Boolean inchild) {
+extern List *forkexec(char *file, List *list, Boolean parent) {
 	int pid, status;
 	Vector *env;
 	gcdisable(0);
 	env = mkenv();
-	pid = efork(!inchild, FALSE);
+	pid = efork(parent, FALSE, FALSE);
 	if (pid == 0) {
 		execve(file, vectorize(list)->vector, env->vector);
-		failexec(file, list);
+		uerror(file);
+		exit(1);
 	}
 	gcenable();
 	status = ewaitfor(pid);
@@ -46,7 +31,7 @@ static List *forkexec(char *file, List *list, Boolean inchild) {
 /* bindargs -- bind an argument list to the parameters of a lambda */
 static Binding *bindargs(Tree *params, List *args, Binding *binding) {
 	if (params == NULL)
-		return mkbinding("*", args, binding);
+		return bind("*", args, binding);
 
 	gcdisable(0);
 
@@ -65,7 +50,7 @@ static Binding *bindargs(Tree *params, List *args, Binding *binding) {
 			value = mklist(args->term, NULL);
 			args = args->next;
 		}
-		binding = mkbinding(param->u[0].s, value, binding);
+		binding = bind(param->u[0].s, value, binding);
 	}
 
 	Ref(Binding *, result, binding);
@@ -73,25 +58,16 @@ static Binding *bindargs(Tree *params, List *args, Binding *binding) {
 	RefReturn(result);
 }
 
-/* pathsearch -- evaluate fn %pathsearch + some argument */
-extern List *pathsearch(Term *term) {
-	List *search, *list;
-	search = varlookup("fn-%pathsearch", NULL);
-	if (search == NULL)
-		fail("es:pathsearch", "%E: fn %%pathsearch undefined", term);
-	list = mklist(term, NULL);
-	list = eval(append(search, list), NULL, 0);
-	return list;
-}
-
 /* eval -- evaluate a list, producing a list */
-extern List *eval(List *list, Binding *binding0, int flags) {
+extern List *eval(List *list, Binding *binding0, Boolean parent) {
 	Closure *cp;
 	List *fn;
+	char *file;
 
 	Ref(char *, funcname, NULL);
 
 restart:
+	debug("<< eval : %L >>\n", list, " ");
 	if (list == NULL) {
 		RefPop(funcname);
 		return true;
@@ -105,16 +81,17 @@ restart:
 		switch (cp->tree->kind) {
 		case nPrim:
 			assert(cp->binding == NULL);
-			lp = prim(cp->tree->u[0].s, lp->next, flags);
+			lp = prim(cp->tree->u[0].s, lp->next, parent);
 			break;
 		case nThunk:
-			lp = walk(cp->tree->u[0].p, cp->binding, flags);
+			lp = walk(cp->tree->u[0].p, cp->binding, parent);
 			break;
 		case nLambda: {
-			Push p;
-			List *e;
 			Handler h;
+			List *e;
 			if ((e = pushhandler(&h)) != NULL) {
+				if (funcname != NULL)
+					varpop("0");
 				if (e->term->str != NULL && streq(e->term->str, "return")) {
 					lp = e->next;
 					goto done;
@@ -124,10 +101,8 @@ restart:
 			Ref(Tree *, tree, cp->tree);
 			Ref(Binding *, context, bindargs(tree->u[0].p, lp->next, cp->binding));
 			if (funcname != NULL)
-				varpush(&p, "0", mklist(mkterm(funcname, NULL), NULL));
-			lp = walk(tree->u[1].p, context, flags);
-			if (funcname != NULL)
-				varpop(&p);
+				varpush("0", mklist(mkterm(funcname, NULL), NULL));
+			lp = walk(tree->u[1].p, context, parent);
 			RefEnd2(context, tree);
 			pophandler(&h);
 			break;
@@ -143,13 +118,6 @@ restart:
 		goto done;
 	}
 
-	/* the logic here is duplicated in $&whatis */
-
-	if (isabsolute(lp->term->str)) {
-		lp = forkexec(lp->term->str, lp, flags & eval_inchild);
-		goto done;
-	}
-
 	fn = varlookup2("fn-", lp->term->str);
 	if (fn != NULL) {
 		funcname = lp->term->str;
@@ -158,84 +126,44 @@ restart:
 		goto restart;
 	}
 
-	fn = pathsearch(lp->term);
-	if (fn != NULL && fn->next == NULL && fn->term->closure == NULL) {
-		char *name = fn->term->str;
-		lp = forkexec(name, lp, flags & eval_inchild);
-		goto done;
-	}
-
-	list = append(fn, lp->next);
-	RefPop2(binding, lp);
-	goto restart;
+	file = which(lp->term->str, TRUE);
+	lp = (file == NULL) ? false : forkexec(file, lp, parent);
 
 done:
-	if ((flags & eval_exitonfalse) && !istrue(lp))
-		exit(exitstatus(lp));
 	list = lp;
 	RefEnd3(binding, lp, funcname);
 	return list;
 }
 
 /* eval1 -- evaluate a term, producing a list */
-extern List *eval1(Term *term, int flags) {
-	return eval(mklist(term, NULL), NULL, flags);
+extern List *eval1(Term *term, Boolean parent) {
+	return eval(mklist(term, NULL), NULL, parent);
 }
 
-/* local -- perform a single local assignment & recurse	or just dispatch a binding */
-static List *local(Tree *defn, Tree *body, Binding *binding, int evalflags) {
-	Push p;
-	char *name;
-	List *list;
-
-	for (;; defn = defn->u[1].p) {
-		if (defn == NULL)
-			return walk(body, binding, evalflags);
-		assert(defn->kind == nList);
-		if (defn->u[0].p != NULL)
-			break;
-	}
-
-	Ref(List *, result, NULL);
-	RefAdd3(defn, body, binding);
-	Ref(Tree *, assign, defn->u[0].p);
-	assert(assign->kind == nAssign);
-	Ref(char *, var, varname(glom(assign->u[0].p, binding, FALSE)));
-	list = glom(assign->u[1].p, binding, TRUE);
-	name = var;
-	RefEnd2(var, assign);
-
-	varpush(&p, name, list);
-	result = local(defn->u[1].p, body, binding, evalflags);
-	varpop(&p);
-
-	RefRemove3(binding, body, defn);
-	RefReturn(result);
-}
-
-
 /* walk -- walk through a tree, evaluating nodes */
-extern List *walk(Tree *tree0, Binding *binding0, int flags) {
-	Tree *volatile tree = tree0;
-	Binding *volatile binding = binding0;
+extern List *walk(Tree *tree, Binding *binding, Boolean parent) {
 
 	SIGCHK();
 
+	debug("<< walk : %T >>\n", tree);
+
 top:
-	if (tree == NULL)
+	if (tree == NULL) {
+		if (!parent)
+			exit(0);
 		return true;
+	}
 
 	switch (tree->kind) {
 
 	case nConcat: case nList: case nQword: case nVar: case nVarsub: case nWord:
-	case nThunk: case nLambda: case nCall: case nPrim: {
+	case nThunk: case nLambda: case nCall: case nPrim: case nRec: {
 		List *list;
 		Ref(Binding *, bp, binding);
 		list = glom(tree, binding, TRUE);
 		binding = bp;
 		RefEnd(bp);
-		list = eval(list, binding, flags);
-		return list;
+		return eval(list, binding, parent);
 	}
 
 	case nMatch: {
@@ -260,7 +188,7 @@ top:
 		return true;
 	}
 
-	case nLet: case nClosure: {
+	case nLocal: {
 		Ref(Binding *, bp, binding);
 		Ref(Tree *, body, tree->u[1].p);
 		Ref(Tree *, defn, tree->u[0].p);
@@ -272,7 +200,7 @@ top:
 			assert(assign->kind == nAssign);
 			Ref(char *, var, varname(glom(assign->u[0].p, bp, FALSE)));
 			Ref(List *, lp, glom(assign->u[1].p, bp, TRUE));
-			bp = mkbinding(var, lp, bp);
+			bp = bind(var, lp, bp);
 			RefEnd3(lp, var, assign);
 		}
 		RefEnd(defn);
@@ -282,83 +210,97 @@ top:
 		goto top;
 	}
 
-	case nLocal:
-		return local(tree->u[0].p, tree->u[1].p, binding, flags);
-
-	case nFor: {
+	case nLet: {
 		Handler h;
 		List *e;
 
+		Ref(List *, status, NULL);
+		Ref(Tree *, tp, tree);
+		Ref(Binding *, bp, binding);
+		Ref(StrList *, vars, NULL);
 		if ((e = pushhandler(&h)) != NULL) {
-			if (e->term->str != NULL && streq(e->term->str, "break"))
-				return e->next;
+			for (; vars != NULL; vars = vars->next)
+				varpop(vars->str);
 			throw(e);
 		}
-
-		Ref(List *, result, true);
-		Ref(Binding *, outer, binding);
-		Ref(Tree *, body, tree->u[1].p);
-		Ref(Binding *, looping, NULL);
-		Ref(Tree *, defn, tree->u[0].p);
-
+		Ref(Tree *, defn, tp->u[0].p);
 		for (; defn != NULL; defn = defn->u[1].p) {
 			assert(defn->kind == nList);
 			if (defn->u[0].p == NULL)
 				continue;
 			Ref(Tree *, assign, defn->u[0].p);
 			assert(assign->kind == nAssign);
-			Ref(char *, var, varname(glom(assign->u[0].p, outer, FALSE)));
-			Ref(List *, lp, glom(assign->u[1].p, outer, TRUE));
-			looping = mkbinding(var, lp, looping);
-			RefEnd3(lp, var, assign);
-			SIGCHK();
+			Ref(char *, var, varname(glom(assign->u[0].p, bp, FALSE)));
+			varpush(var, glom(assign->u[1].p, bp, TRUE));
+			vars = mkstrlist(var, vars);
+			RefEnd2(var, assign);
 		}
 		RefEnd(defn);
+		status = walk(tp->u[1].p, bp, parent);
+		for (; vars != NULL; vars = vars->next)
+			varpop(vars->str);
+		pophandler(&h);
+		RefEnd3(vars, bp, tp);
+		RefReturn(status);
+	}
 
-		if (looping != NULL) {
-			Binding *prev, *next;
-			prev = NULL;
-			do {
-				next = looping->next;
-				looping->next = prev;
-				prev = looping;
-			} while ((looping = next) != NULL);
-			looping = prev;
+	case nFor: {
+		Handler h;
+		List *e;
+
+		if ((e = pushhandler(&h)) != NULL) {
+			if (e->term->str != NULL && streq(e->term->str, "return"))
+				return e->next;
+			throw(e);
 		}
 
+		Ref(List *, result, true);
+		Ref(Tree *, body, tree->u[1].p);
+		Ref(Binding *, context, binding);
+		Ref(Binding *, looping, NULL);
+		Ref(Tree *, defn, tree->u[0].p);
+		Ref(Binding *, oldbinding, binding);
+		
+		for (; defn != NULL; defn = defn->u[1].p) {
+			Term placeholder;
+			assert(defn->kind == nList);
+			if (defn->u[0].p == NULL)
+				continue;
+			Ref(Tree *, assign, defn->u[0].p);
+			assert(assign->kind == nAssign);
+			Ref(char *, var, varname(glom(assign->u[0].p, oldbinding, FALSE)));
+			Ref(List *, lp, glom(assign->u[1].p, oldbinding, TRUE));
+			looping = bind(var, lp, looping);
+			context = bind(var, mklist(&placeholder, NULL), context);
+			RefEnd3(lp, var, assign);
+		}
+		RefEnd2(oldbinding, defn);
+
 		for (;;) {
 			Boolean allnull = TRUE;
-			Ref(Binding *, bp, outer);
 			Ref(Binding *, lp, looping);
-			for (; lp != NULL; lp = lp->next) {
-				Ref(List *, defn, NULL);
-				if (lp->defn != NULL) {
-					defn = mklist(lp->defn->term, NULL);
+			Ref(Binding *, bp, context);
+			for (; lp != NULL; lp = lp->next, bp = bp->next)
+				if (lp->defn == NULL)
+					bp->defn = NULL;
+				else {
+					bp->defn->term = lp->defn->term;
 					lp->defn = lp->defn->next;
 					allnull = FALSE;
 				}
-				bp = mkbinding(lp->name, defn, bp);
-				RefEnd(defn);
-			}
-			RefEnd(lp);
-			if (allnull) {
-				RefPop(bp);
+			RefEnd2(bp, lp);
+			if (allnull)
 				break;
-			}
-			result = walk(body, bp, flags & eval_exitonfalse);
-			RefEnd(bp);
-			SIGCHK();
+			result = walk(body, context, TRUE);
 		}
 
 		e = result;
-		RefEnd4(looping, body, outer, result);
+		RefEnd4(looping, context, body, result);
 		pophandler(&h);
 		return e;
 	}
-	
+
 	default:
 		panic("walk: bad node kind %d", tree->kind);
-
 	}
-	NOTREACHED;
 }
