@@ -1,20 +1,71 @@
-/* glob.c -- wildcard matching ($Revision: 1.5 $) */
-
-#define	REQUIRE_STAT	1
-#define	REQUIRE_DIRENT	1
+/* glob.c -- wildcard matching */
 
 #include "es.h"
-#include "gc.h"
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#if USE_DIRENT
+#include <dirent.h>
+typedef struct dirent Dirent;
+#else
+#include <sys/dir.h>
+typedef struct direct Dirent;
+#endif
+
+extern void *qsort(void *base, size_t nmemb, size_t size, int (*compar)(const void *, const void *));
+
+/* prototypes for XXXdir functions. comment out if necessary */
+extern DIR *opendir(const char *);
+extern Dirent *readdir(DIR *);
+/*extern int closedir(DIR *);*/
 
 char QUOTED[] = "QUOTED", UNQUOTED[] = "RAW";
 
-/* hastilde -- true iff the first character is a ~ and it is not quoted */
-static Boolean hastilde(const char *s, const char *q) {
-	return *s == '~' && (q == UNQUOTED || *q == 'r');
+/* qstrcmp -- a strcmp wrapper for qsort */
+static int qstrcmp(const void *s1, const void *s2) {
+	return strcmp(*(const char **)s1, *(const char **)s2);
 }
 
-/* haswild -- true iff some unquoted character is a wildcard character */
-static Boolean haswild(const char *s, const char *q) {
+/* listsort */
+static List *listsort(List *list) {
+	if (length(list) > 1) {
+		Vector *v = vectorize(list);
+		qsort(v->vector, v->len, sizeof (char *), qstrcmp);
+		gcdisable(0);
+		Ref(List *, lp, listify(v->len, v->vector));
+		gcenable();
+		list = lp;
+		RefEnd(lp);
+	}
+	return list;
+}
+
+/* dirmatch -- match a pattern against the contents of directory */
+static List *dirmatch(const char *prefix, const char *dirname, const char *pattern, const char *quote) {
+	List *list, **prevp;
+	static DIR *dirp;
+	static Dirent *dp;
+	static struct stat s;
+
+	/* TODO: optimize case where there are no meta chars */
+
+	assert(gcblocked > 0);
+
+	/* opendir succeeds on regular files on some systems, so the stat() call is necessary (sigh) */
+	if (stat(dirname, &s) < 0 || (s.st_mode & S_IFMT) != S_IFDIR || (dirp = opendir(dirname)) == NULL)
+		return NULL;	
+	for (list = NULL, prevp = &list; (dp = readdir(dirp)) != NULL;)
+		if (match(dp->d_name, pattern, quote) && (*dp->d_name != '.' || *pattern == '.')) {
+			List *lp = mklist(mkterm(str("%s%s", prefix, dp->d_name), NULL), NULL);
+			*prevp = lp;
+			prevp = &lp->next;
+		}
+	closedir(dirp);
+	return list;
+}
+
+/* hasmeta -- true iff some unquoted character is a meta character */
+static Boolean hasmeta(const char *s, const char *q) {
 	if (q == QUOTED)
 		return FALSE;
 	if (q == UNQUOTED)
@@ -34,35 +85,6 @@ static Boolean haswild(const char *s, const char *q) {
 	}
 }
 
-/* dirmatch -- match a pattern against the contents of directory */
-static List *dirmatch(const char *prefix, const char *dirname, const char *pattern, const char *quote) {
-	List *list, **prevp;
-	static DIR *dirp;
-	static Dirent *dp;
-	static struct stat s;
-
-	if (!haswild(pattern, quote)) {
-		char *name = str("%s%s", prefix, pattern);
-		if (lstat(name, &s) == -1)
-			return NULL;
-		return mklist(mkterm(name, NULL), NULL);
-	}
-
-	assert(gcisblocked());
-
-	/* opendir succeeds on regular files on some systems, so the stat() call is necessary (sigh) */
-	if (stat(dirname, &s) == -1 || (s.st_mode & S_IFMT) != S_IFDIR || (dirp = opendir(dirname)) == NULL)
-		return NULL;	
-	for (list = NULL, prevp = &list; (dp = readdir(dirp)) != NULL;)
-		if (match(dp->d_name, pattern, quote) && (*dp->d_name != '.' || *pattern == '.')) {
-			List *lp = mklist(mkterm(str("%s%s", prefix, dp->d_name), NULL), NULL);
-			*prevp = lp;
-			prevp = &lp->next;
-		}
-	closedir(dirp);
-	return list;
-}
-
 /* listglob -- glob a directory plus a filename pattern into a list of names */
 static List *listglob(List *list, char *pattern, char *quote, size_t slashcount) {
 	List *result, **prevp;
@@ -147,8 +169,7 @@ static List *glob1(const char *pattern, const char *quote) {
 			*p++ = *s++, *qp++ = *q++; /* get pat */
 		*p = '\0';
 		matched = listglob(matched, pat, qpat, slashcount);
-	} while (*s != '\0' && matched != NULL);
-
+	} while (*s != '\0');
 	return matched;
 }
 
@@ -159,114 +180,38 @@ static List *glob0(List *list, StrList *quote) {
 	for (result = NULL, prevp = &result; list != NULL; list = list->next, quote = quote->next)
 		if (
 			quote->str == QUOTED
-			|| !haswild(list->term->str, quote->str)
+			|| !hasmeta(list->term->str, quote->str)
 			|| (expand1 = glob1(list->term->str, quote->str)) == NULL
 		) {
 			*prevp = list;
 			prevp = &list->next;
 		} else {
-			*prevp = sortlist(expand1);
+			*prevp = listsort(expand1);
 			while (*prevp != NULL)
 				prevp = &(*prevp)->next;
 		}
 	return result;
 }
 
-/* expandhome -- do tilde expansion by calling fn %home */
-static char *expandhome(char *s, StrList *qp) {
-	int c;
-	size_t slash;
-	List *fn = varlookup("fn-%home", NULL);
-
-	assert(*s == '~');
-	assert(qp->str == UNQUOTED || *qp->str == 'r');
-
-	if (fn == NULL)
-		return s;
-
-	for (slash = 1; (c = s[slash]) != '/' && c != '\0'; slash++)
-		;
-
-	Ref(char *, string, s);
-	Ref(StrList *, quote, qp);
-	Ref(List *, list, NULL);
-	RefAdd(fn);
-	if (slash > 1)
-		list = mklist(mkterm(gcndup(s + 1, slash - 1), NULL), NULL);
-	RefRemove(fn);
-
-	list = eval(append(fn, list), NULL, 0);
-
-	if (list != NULL) {
-		if (list->next != NULL)
-			fail("es:expandhome", "%%home returned more than one value");
-		Ref(char *, home, getstr(list->term));
-		if (c == '\0') {
-			string = home;
-			quote->str = QUOTED;
-		} else {
-			char *q;
-			size_t pathlen = strlen(string);
-			size_t homelen = strlen(home);
-			size_t len = pathlen - slash + homelen;
-			s = gcalloc(len + 1, &StringTag);
-			memcpy(s, home, homelen);
-			memcpy(&s[homelen], &string[slash], pathlen - slash);
-			s[len] = '\0';
-			string = s;
-			q = quote->str;
-			if (q == UNQUOTED) {
-				q = gcalloc(len + 1, &StringTag);
-				memset(q, 'q', homelen);
-				memset(&q[homelen], 'r', pathlen - slash);
-				q[len] = '\0';
-			} else if (strchr(q, 'r') == NULL)
-				q = QUOTED;
-			else {
-				q = gcalloc(len + 1, &StringTag);
-				memset(q, 'q', homelen);
-				memcpy(&q[homelen], &quote->str[slash], pathlen - slash);
-				q[len] = '\0';
-			}
-			quote->str = q;
-		}
-		RefEnd(home);
-	}
-	RefEnd2(list, quote);
-	RefReturn(string);
-}
-
-/* glob -- globbing prepass (glob if we need to, and dispatch for tilde expansion) */
+/* glob -- globbing prepass (glob if we need to) */
 extern List *glob(List *list, StrList *quote) {
 	List *lp;
 	StrList *qp;
-	Boolean doglobbing = FALSE;
+
+	debug("<< glob : %L >>\n", list, " ");
+	debug("`` glob : %Z ''\n", quote, " ");
 
 	for (lp = list, qp = quote; lp != NULL; lp = lp->next, qp = qp->next)
 		if (qp->str != QUOTED) {
 			assert(lp->term != NULL && lp->term->str != NULL);
 			assert(qp->str == UNQUOTED || strlen(qp->str) == strlen(lp->term->str));
-			if (hastilde(lp->term->str, qp->str)) {
-				Ref(List *, l0, list);
-				Ref(List *, lr, lp);
-				Ref(StrList *, q0, quote);
-				Ref(StrList *, qr, qp);
-				lr->term->str = expandhome(lr->term->str, qp);
-				lp = lr;
-				qp = qr;
-				list = l0;
-				quote = q0;
-				RefEnd4(qr, q0, lr, l0);
+			if (hasmeta(lp->term->str, qp->str)) {
+				gcdisable(0);
+				list = glob0(list, quote);
+				Ref(List *, result, list);
+				gcenable();
+				RefReturn(result);
 			}
-			if (haswild(lp->term->str, qp->str))
-				doglobbing = TRUE;
 		}
-
-	if (!doglobbing)
-		return list;
-	gcdisable(0);
-	list = glob0(list, quote);
-	Ref(List *, result, list);
-	gcenable();
-	RefReturn(result);
+	return list;
 }
