@@ -1,4 +1,4 @@
-/* print.c -- formatted printing routines ($Revision: 1.1.1.1 $) */
+/* print.c -- formatted printing routines (Paul Haahr, 12/91) */
 
 #include "es.h"
 #include "print.h"
@@ -10,7 +10,7 @@
  *	true return -> flag changes only, not a conversion
  */
 
-#define	Flag(name, flag) \
+#define Flag(name, flag) \
 static Boolean name(Format *format) { \
 	format->flags |= flag; \
 	return TRUE; \
@@ -18,7 +18,7 @@ static Boolean name(Format *format) { \
 
 Flag(uconv,	FMT_unsigned)
 Flag(hconv,	FMT_short)
-Flag(longconv,	FMT_long)
+Flag(lconv,	FMT_long)
 Flag(altconv,	FMT_altform)
 Flag(leftconv,	FMT_leftside)
 Flag(dotconv,	FMT_f2set)
@@ -41,8 +41,8 @@ static Boolean zeroconv(Format *format) {
 	return TRUE;
 }
 
-static void pad(Format *format, long len, int c) {
-	while (len-- > 0)
+static void pad(Format *format, size_t len, int c) {
+	while (len-- != 0)
 		fmtputc(format, c);
 }
 
@@ -89,6 +89,8 @@ static void intconv(Format *format, unsigned int radix, int upper, char *altform
 	flags = format->flags;
 	if (flags & FMT_long)
 		n = va_arg(format->args, long);
+	else if (flags & FMT_short)
+		n = va_arg(format->args, short);
 	else
 		n = va_arg(format->args, int);
 
@@ -169,12 +171,10 @@ static Boolean badconv(Format *format) {
  * conversion table management
  */
 
-static Conv *fmttab;
+static Conv fmttab[MAXCONV];
 
 static void inittab(void) {
 	int i;
-
-	fmttab = ealloc(MAXCONV * sizeof (Conv));
 	for (i = 0; i < MAXCONV; i++)
 		fmttab[i] = badconv;
 
@@ -187,7 +187,7 @@ static void inittab(void) {
 
 	fmttab['u'] = uconv;
 	fmttab['h'] = hconv;
-	fmttab['l'] = longconv;
+	fmttab['l'] = lconv;
 	fmttab['#'] = altconv;
 	fmttab['-'] = leftconv;
 	fmttab['.'] = dotconv;
@@ -199,7 +199,7 @@ static void inittab(void) {
 
 Conv fmtinstall(int c, Conv f) {
 	Conv oldf;
-	if (fmttab == NULL)
+	if (fmttab[0] == NULL)
 		inittab();
 	c &= MAXCONV - 1;
 	oldf = fmttab[c];
@@ -263,20 +263,14 @@ extern int printfmt(Format *format, const char *fmt) {
  * the public entry points
  */
 
-extern int fmtprint VARARGS2(Format *, format, const char *, fmt) {
+extern int fmtprint(Format *format, const char *fmt, ...) {
 	int n = -format->flushed;
-#if NO_VA_LIST_ASSIGN
-	va_list saveargs;
-
-	memcpy(saveargs, format->args, sizeof(va_list));
-#else
 	va_list saveargs = format->args;
-#endif
 
-	VA_START(format->args, fmt);
+	va_start(format->args, fmt);
 	n += printfmt(format, fmt);
 	va_end(format->args);
-	va_copy(format->args, saveargs);
+	format->args = saveargs;
 
 	return n + format->flushed;
 }
@@ -289,9 +283,8 @@ static void fprint_flush(Format *format, size_t more) {
 	format->buf = format->bufbegin;
 	while (n != 0) {
 		int written = write(format->u.n, buf, n);
-		if (written == -1) {
-			if (format->u.n != 2)
-				uerror("write");
+		if (written < 0) {
+			uerror("write");
 			exit(1);
 		}
 		n -= written;
@@ -306,45 +299,54 @@ static void fdprint(Format *format, int fd, const char *fmt) {
 	format->bufend	= buf + sizeof buf;
 	format->grow	= fprint_flush;
 	format->flushed	= 0;
-	format->u.n	= fdmap(fd);
+	format->u.n	= fd;
 
-	gcdisable();
+	gcdisable(0);
 	printfmt(format, fmt);
 	fprint_flush(format, 0);
 	gcenable();
 }
 
-extern int fprint VARARGS2(int, fd, const char *, fmt) {
+extern int fprint(int fd, const char *fmt, ...) {
 	Format format;
-	VA_START(format.args, fmt);
+	va_start(format.args, fmt);
 	fdprint(&format, fd, fmt);
 	va_end(format.args);
 	return format.flushed;
 }
 
-extern int print VARARGS1(const char *, fmt) {
+extern int print(const char *fmt, ...) {
 	Format format;
-	VA_START(format.args, fmt);
+	va_start(format.args, fmt);
 	fdprint(&format, 1, fmt);
 	va_end(format.args);
 	return format.flushed;
 }
 
-extern int eprint VARARGS1(const char *, fmt) {
+extern int eprint(const char *fmt, ...) {
 	Format format;
-	VA_START(format.args, fmt);
+	va_start(format.args, fmt);
 	fdprint(&format, 2, fmt);
 	va_end(format.args);
 	return format.flushed;
 }
 
-extern noreturn panic VARARGS1(const char *, fmt) {
+extern noreturn panic(const char *fmt, ...) {
 	Format format;
-	gcdisable();
-	VA_START(format.args, fmt);
-	eprint("es panic: ");
+	gcdisable(0);
+	va_start(format.args, fmt);
 	fdprint(&format, 2, fmt);
 	va_end(format.args);
 	eprint("\n");
 	exit(1);
 }
+
+extern void debug(const char *fmt, ...) {
+	extern Boolean bugme;
+	if (bugme) {
+		Format format;
+		va_start(format.args, fmt);
+		fdprint(&format, 2, fmt);
+		va_end(format.args);
+	}
+}
