@@ -1,4 +1,4 @@
-/* str.c -- es string operations ($Revision: 1.1.1.1 $) */
+/* str.c -- es string operations */
 
 #include "es.h"
 #include "gc.h"
@@ -6,29 +6,24 @@
 
 /* grow -- buffer grow function for str() */
 static void str_grow(Format *f, size_t more) {
-	Buffer *buf = expandbuffer(f->u.p, more);
-	f->u.p		= buf;
-	f->buf		= buf->str + (f->buf - f->bufbegin);
-	f->bufbegin	= buf->str;
-	f->bufend	= buf->str + buf->len;
+	Buffer buf;
+	buf = expandbuffer(more);
+	f->buf		= buf.str;
+	f->bufbegin	= buf.str;
+	f->bufend	= buf.str + buf.len;
 }
 
 /* strv -- print a formatted string into gc space */
 extern char *strv(const char *fmt, va_list args) {
-	Buffer *buf;
+	Buffer buf;
 	Format format;
 
-	gcdisable();
+	gcdisable(0);
 	buf = openbuffer(0);
-	format.u.p	= buf;
-#if NO_VA_LIST_ASSIGN
-	memcpy(format.args, args, sizeof(va_list));
-#else
 	format.args	= args;
-#endif
-	format.buf	= buf->str;
-	format.bufbegin	= buf->str;
-	format.bufend	= buf->str + buf->len;
+	format.buf	= buf.str;
+	format.bufbegin	= buf.str;
+	format.bufend	= buf.str + buf.len;
 	format.grow	= str_grow;
 	format.flushed	= 0;
 
@@ -36,14 +31,14 @@ extern char *strv(const char *fmt, va_list args) {
 	fmtputc(&format, '\0');
 	gcenable();
 
-	return sealbuffer(format.u.p);
+	return sealbuffer(format.buf);
 }
 
 /* str -- create a string (in garbage collection space) by printing to it */
-extern char *str VARARGS1(const char *, fmt) {
+extern char *str(const char *fmt, ...) {
 	char *s;
 	va_list args;
-	VA_START(args, fmt);
+	va_start(args, fmt);
 	s = strv(fmt, args);
 	va_end(args);
 	return s;
@@ -66,10 +61,10 @@ static void mprint_grow(Format *format, size_t more) {
 }
 
 /* mprint -- create a string in ealloc space by printing to it */
-extern char *mprint VARARGS1(const char *, fmt) {
+extern char *mprint(const char *fmt,...) {
 	Format format;
 	format.u.n = 1;
-	VA_START(format.args, fmt);
+	va_start(format.args, fmt);
 
 	format.buf	= ealloc(PRINT_ALLOCSIZE);
 	format.bufbegin	= format.buf;
@@ -89,10 +84,10 @@ extern char *mprint VARARGS1(const char *, fmt) {
  *	to even include these is probably a premature optimization
  */
 
-DefineTag(StrList, static);
+static Tag StrListTag;
 
 extern StrList *mkstrlist(char *str, StrList *next) {
-	gcdisable();
+	gcdisable(0);
 	assert(str != NULL);
 	Ref(StrList *, list, gcnew(StrList));
 	list->str = str;
@@ -114,3 +109,5 @@ static size_t StrListScan(void *p) {
 	return sizeof (StrList);
 }
 
+static DefineTag(StrList);
+
